<?php
/**
 * Copyright (c) 2014 Yubico AB
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * This is a minimal example of U2F registration and authentication.
 * The data that has to be stored between registration and authentication
 * is stored in browser localStorage, so there's nothing real-world
 * about this.
 */
require_once('../../src/u2flib_server/U2F.php');
$scheme = isset($_SERVER['HTTPS']) ? "https://" : "http://";
$u2f = new u2flib_server\U2F($scheme . $_SERVER['HTTP_HOST']);

$request = '';
?>
<!DOCTYPE html>
<html lang="en">
<head>
    <title>PHP U2F Demo</title>

    <!-- script src="../assets/u2f-api.js"></script -->
<?php
function fixupArray($data) {
	$ret = array();
	$decoded = json_decode($data);
	foreach ($decoded as $d) {
		$ret[] = json_encode($d);
	}
	return $ret;
}

if($_SERVER['REQUEST_METHOD'] === 'POST') {
?>
    <script>
		function downloadAPI(callback) {
			console.log('Loading API...');
			const script1 = document.createElement('SCRIPT');
			script1.src = '../assets/u2f-api.js';
			script1.onload = callback;
			document.head.appendChild(script1);
		}

        function addRegistration(reg) {
            var existing = localStorage.getItem('u2fregistration');
            var regobj = JSON.parse(reg);
            var data = null;
            if(existing) {
                data = JSON.parse(existing);
                if(Array.isArray(data)) {
                    for (var i = 0; i < data.length; i++) {
                        if(data[i].keyHandle === regobj.keyHandle) {
                            data.splice(i,1);
                            break;
                        }
                    }
                    data.push(regobj);
                } else {
                    data = null;
                }
            }
            if(data == null) {
                data = [regobj];
            }
            localStorage.setItem('u2fregistration', JSON.stringify(data));
        }

<?php
	if(isset($_POST['startRegister'])) { /* ------- start register --------- */
		$regs = json_decode($_POST['registrations']) ? : array();
		list($appId, $regRequest, $regRegisteredKeys)  = $u2f->getRegisterData($regs);
		$request = json_encode($regRequest, JSON_UNESCAPED_SLASHES);
?>
		function callRegister() {
	        setTimeout(function() {
	            console.log("Register: ", request);
	            u2f.register(
		            '<?= $appId ?>',
					[<?= $request ?>],
					<?php echo json_encode($regRegisteredKeys); ?>,
					function(data) {
		                console.log("Register callback", data);
		                if(data.errorCode && data.errorCode != 0) {
		                    alert("registration failed with errror: " + data.errorCode);
		                    return;
		                }
		                var reg = document.getElementById('doRegister');
		                reg.value=JSON.stringify(data);
		                reg.form.submit();
		            }
	            );
	        }, 1000);
		}

		if(typeof u2f === 'undefined') {
			downloadAPI(callRegister)
		} else {
			callRegister();
		}
<?php
	} elseif($_POST['doRegister']) { /* -------- Do register ---------- */
		try {
			$data = $u2f->doRegister(
				json_decode($_POST['request']),
				json_decode($_POST['doRegister'])
			);
			echo "var registration = '" . json_encode($data) . "';\n";
?>
        addRegistration(registration);
        alert("registration successful!");
<?php
                } catch(u2flib_server\Error $e) {
                    echo "alert('error:" . $e->getMessage() . "');\n";
                }
	} elseif(isset($_POST['startAuthenticate'])) { /* -------------- start authenticate --------- */
		$regs = json_decode($_POST['registrations']);
		$data = $u2f->getAuthenticateData($regs);
		list($appId, $request, $signRegisteredKeys) = $u2f->getAuthenticateData($regs);
?>
        function callAuthenticate() {
	        setTimeout(function() {
	            console.log("sign: '<?= $request ?>'");
	            u2f.sign(
					'<?= $appId ?>',
					'<?= $request ?>',
					<?php echo json_encode($signRegisteredKeys); ?>,
					function(data) {
		                console.log("Authenticate callback", data);
		                var reg = document.getElementById('doAuthenticate');
		                reg.value = JSON.stringify(data);
		                reg.form.submit();
		            }
	            );
	        }, 1000);
		}

		if(typeof u2f === 'undefined') {
			downloadAPI(callAuthenticate)
		} else {
			callAuthenticate();
		}
<?php
	} elseif($_POST['doAuthenticate']) { /* ---------- Do authenticate ----------- */
		try {
			$data = $u2f->doAuthenticate(
				$_POST['request'],
				json_decode($_POST['registrations']),
				json_decode($_POST['doAuthenticate'])
			);
			echo "var registration = '" . json_encode($data) . "';\n";
			echo "addRegistration(registration);\n";
			echo "alert('Authentication successful, counter:" . $data->counter . "');\n";
		} catch(u2flib_server\Error $e) {
			echo "alert('error:" . $e->getMessage() . "');\n";
		}
	}
?>
    </script>
<?php
}
?>
</head>
<body>
<form method="POST">
    <input type="submit" name="startRegister" value="Register" />
    <input type="hidden" name="doRegister" id="doRegister"/>
    <input type="submit" name="startAuthenticate" id="startAuthenticate" value="Authenticate" />
    <input type="hidden" name="doAuthenticate" id="doAuthenticate"/>
    <input type="hidden" name="request" id="request" value='<?= $request ?>'/>
    <input type="hidden" name="registrations" id="registrations"/>
</form>

<p>
    <span id="registered">0</span> Authenticators currently registered.
</p>

<script>
    var reg = localStorage.getItem('u2fregistration');
    var auth = document.getElementById('startAuthenticate');
    if(reg == null) {
        auth.disabled = true;
    } else {
        var regs = document.getElementById('registrations');
        decoded = JSON.parse(reg);
        if(!Array.isArray(decoded)) {
            auth.disabled = true;
        } else {
            regs.value = reg;
            console.log("localStorage for the registrations are setting to : ", reg);
            var regged = document.getElementById('registered');
            regged.innerHTML = decoded.length;
        }
    }
</script>
</body>
</html>
